/* Agar icon reference file */
/* Generated by agarpaint 1.1 */

#include <agar/vg/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticIcon vgIconPoints;
extern DECLSPEC AG_StaticIcon vgIconLine;
extern DECLSPEC AG_StaticIcon vgIconCircle;
extern DECLSPEC AG_StaticIcon vgIconBezier;
extern DECLSPEC AG_StaticIcon vgIconBlock;
extern DECLSPEC AG_StaticIcon vgIconText;
extern DECLSPEC AG_StaticIcon vgIconSnapFree;
extern DECLSPEC AG_StaticIcon vgIconSnapRint;
extern DECLSPEC AG_StaticIcon vgIconSnapGrid;
extern DECLSPEC AG_StaticIcon vgIconSnapEndpt;
extern DECLSPEC AG_StaticIcon vgIconSnapEndptDist;
extern DECLSPEC AG_StaticIcon vgIconSnapClosest;
extern DECLSPEC AG_StaticIcon vgIconSnapCenterPt;
extern DECLSPEC AG_StaticIcon vgIconSnapMiddlePt;
extern DECLSPEC AG_StaticIcon vgIconSnapIntsectAuto;
extern DECLSPEC AG_StaticIcon vgIconSnapIntsectManual;
extern DECLSPEC AG_StaticIcon vgIconPolygon;
extern DECLSPEC AG_StaticIcon vgIconDrawing;
extern DECLSPEC AG_StaticIcon vgIconProximity;
extern DECLSPEC AG_StaticIcon vgIconSelectArrow;
__END_DECLS
/* Close generated block */
#include <agar/vg/close.h>
