/*	Public domain	*/

/* Global Agar build options */

#include <agar/config/ag_ansi_color.h>
#include <agar/config/ag_debug.h>
#include <agar/config/ag_enable_dso.h>
#include <agar/config/ag_enable_exec.h>
#include <agar/config/ag_enable_string.h>
#include <agar/config/ag_event_loop.h>
#include <agar/config/ag_legacy.h>
#include <agar/config/ag_named_args.h>
#include <agar/config/ag_namespaces.h>
#include <agar/config/ag_serialization.h>
#include <agar/config/ag_threads.h>
#include <agar/config/ag_timers.h>
#include <agar/config/ag_type_safety.h>
#include <agar/config/ag_unicode.h>
#include <agar/config/ag_user.h>
#include <agar/config/ag_verbosity.h>
#include <agar/config/ag_widgets.h>
#include <agar/config/ag_wm_hints.h>
