/*	Public domain	*/

#ifndef _AGAR_GUI_PUBLIC_H_
#define _AGAR_GUI_PUBLIC_H_
#define _AGAR_GUI_PUBLIC

#include <agar/core/core.h>

#include <agar/gui/gui.h>
#include <agar/gui/widget.h>
#include <agar/gui/window.h>
#include <agar/gui/text.h>
#include <agar/gui/icons.h>
#include <agar/gui/iconmgr.h>
#include <agar/gui/primitive.h>

#include <agar/gui/box.h>
#include <agar/gui/button.h>
#include <agar/gui/checkbox.h>
#include <agar/gui/combo.h>
#include <agar/gui/console.h>
#include <agar/gui/dir_dlg.h>
#include <agar/gui/editable.h>
#include <agar/gui/file_dlg.h>
#include <agar/gui/fixed.h>
#include <agar/gui/font_selector.h>
#include <agar/gui/fixed_plotter.h>
#include <agar/gui/glview.h>
#include <agar/gui/graph.h>
#include <agar/gui/hsvpal.h>
#include <agar/gui/icon.h>
#include <agar/gui/keymap.h>
#include <agar/gui/label.h>
#include <agar/gui/menu.h>
#include <agar/gui/mpane.h>
#include <agar/gui/mfspinbutton.h>
#include <agar/gui/mspinbutton.h>
#include <agar/gui/notebook.h>
#include <agar/gui/numerical.h>
#include <agar/gui/objsel.h>
#include <agar/gui/pane.h>
#include <agar/gui/pixmap.h>
#include <agar/gui/progress_bar.h>
#include <agar/gui/radio.h>
#include <agar/gui/scrollbar.h>
#include <agar/gui/scrollview.h>
#include <agar/gui/separator.h>
#include <agar/gui/socket.h>
#include <agar/gui/slider.h>
#include <agar/gui/statusbar.h>
#include <agar/gui/table.h>
#include <agar/gui/treetbl.h>
#include <agar/gui/textbox.h>
#include <agar/gui/text_cache.h>
#include <agar/gui/tlist.h>
#include <agar/gui/toolbar.h>
#include <agar/gui/ucombo.h>
#include <agar/gui/units.h>

#include <agar/gui/load_surface.h>

#ifdef __APPLE__
#include <agar/gui/sdl.h>
#endif

#endif /* _AGAR_GUI_PUBLIC_H_ */
