/*	Public domain	*/

#ifndef _AGAR_MAP_RG_PUBLIC_H_
#define _AGAR_MAP_RG_PUBLIC_H_
#define _AGAR_MAP_RG_PUBLIC

#include <agar/core.h>
#include <agar/gui.h>

#include <agar/math/m.h>
#include <agar/math/m_gui.h>

#include <agar/map/rg_tileset.h>
#include <agar/map/rg_tileview.h>
#include <agar/map/rg_icons.h>

#endif /* _AGAR_MAP_RG_PUBLIC_H_ */
